<?php
defined( 'ABSPATH' ) or exit;   // show a blank page if try to access this file directly

require_once( EWZ_PLUGIN_DIR . "classes/ewz-webform.php");
require_once( EWZ_PLUGIN_DIR . "classes/validation/ewz-input.php");

/* Validation for the forms on Webforms page */

class Ewz_Webform_Set_Input extends Ewz_Input
{

     function __construct( $form_data ) {
         parent::__construct( $form_data );
         assert( is_array( $form_data ) );
       
         $this->rules = array(

                  '_wp_http_referer' => array( 'type' => 'to_string', 'req' => false, 'val' => '' ),
                  'ewzmode'          => array( 'type' => 'fixed',     'req' => true,  'val' => 'wf_set' ),
                  'action'           => array( 'type' => 'limited',   'req' => true,  'val' => ['ewz_webforms_batch_delete', 
                                                                                                'ewz_save_webform_order',
                                                                                                'ewz_hide_webforms',
                                                                                                'ewz_remove_zipfiles']),
                  'show_hide'        => array( 'type' => 'to_bool',   'req' => false, 'val' => '' ),
                  'ewznonce'         => array( 'type' => 'anonce',    'req' => true,  'val' => '' ),
                  'wforder'          => array( 'type' => 'v_order',   'req' => false, 'val' => ['len' => 10] ),
                  'webform_ids'      => array( 'type' => 'v_wfids',   'req' => false, 'val' => ['len' => 10] ),
                  'target'           => array( 'type' => 'limited',   'req' => false, 'val' => ['forms','items','zipfiles']),
                 );
        $this->validate();
     }

     function validate( ){

        // an unchecked checkbox does not create any matching value in $_POST
        if ( !array_key_exists( 'show_hide', $this->input_data ) ) {
            $this->input_data['show_hide'] = false;
        }
        
        parent::validate();

        return true;
     }

    function v_order( $value, $arg ) {
        assert( is_array( $value ) );
        assert( isset( $arg ) );
        if ( is_array( $value ) ) {
            $seen = array();
            $count = count($value);
            foreach ( $value as $key => $nm ) {
                self::to_seq( $key, $arg);
                if ( !preg_match( '/^\d+$/', $nm ) ) {
                    throw new EWZ_Exception( "Invalid order $nm" );
                }
                if( $nm < 0 || $nm >= $count ){
                    throw new EWZ_Exception( "Invalid value for order $nm" );
                } 
                if( isset($seen[$nm]) ){
                    throw new EWZ_Exception( "Duplicate value for order $nm" );
                }
                $seen[$nm] = true;
            }   
        } else {
            throw new EWZ_Exception( "Bad input data for webform order" );
        }
        return true;
     }


  function v_wfids( &$value, $arg ) {
        assert( is_array( $value ) || empty( $value ) );
        assert( isset( $arg ));
        if ( is_array( $value ) ) {
            $seen = array();
            foreach ( $value as $key => $nm ) {
                if ( !is_int( $key ) ) {
                    throw new EWZ_Exception( "Invalid key $key" );
                }
                self::to_seq($nm, $arg);
                $value[$key] = intval( $nm );
                if( isset($seen[$key]) ){
                    throw new EWZ_Exception( "Duplicate value for key $nm" );
                }
                $seen[$key] = true;
            }   
        } else {
            throw new EWZ_Exception( "Bad input data for webform batch delete" );
        }
        return true;
     }
}
